package wesen.robot;

import java.awt.*;
import java.io.*;
import java.util.*;
import wesen.util.*;

/**
 * Der Aktor ist statisch und wird waehrend der Simulation nicht bewegt, er kann aber
 * ein Licht haben welches auf die Roboter einwirkt.
 * 
 * @author Wesen-Team
 * @version 1.0
 * @since 04.07.2001
 */
public class Actor extends AnimationObject {

  /**
   * Ueberschreiben der serialisierungs-ID, damit werden InvalidClassException ausgeschlossen.
   * Besser waere alle nicht fuer die Serialsierung benutzten Variablen als transient zu 
   * deklarieren, dann wuerde bei einer Aenderung der nicht relevanten Variablen dieses 
   * Problem nicht auftreten.
   */
  static final long serialVersionUID=5229886082550730614L;

  /**
   * Durchmesser des Kreises (Aktor)
   */
  private static final int  DIAMETER = 30;

  /**
   * Hat der Aktor ein Licht
   */
  private boolean           hasLight;

  /**
   * Kreiert einen Aktor.
   */
  public Actor() {}

  /**
   * Kreiert einen Aktor welcher normalwerweise vom Benutzer interaktiv ins "Leben"
   * gerufen wurde.
   * 
   * @param originX Position in der X-Achse
   * @param originY Position in der Y-Achse
   * @param fileName Name der Datei welche die erweiterten Eigenschaften des Aktoren
   * enthaelt.
   * @param path Path wo sich die Datei befindet
   */
  public Actor(int originX, int originY, String fileName, String path) {
    setBaseValues(originX, originY, fileName, path);
  }

  /**
   * Setzen der Eigenschaften, diese werden zu einem Teil aus einer Datei gelesen
   *
   * @param originX Position in der X-Achse
   * @param originY Position in der Y-Achse
   * @param fileName Name der Datei
   * @param path Path wo sich die Datei befindet
   */
  private void setBaseValues(int originX, int originY, String fileName, 
                             String path) {
    diameter = DIAMETER;
    this.originX = this.x = originX;
    this.originY = this.y = originY;
    this.fileName = fileName;
    


    try {
      /*  
      System.out.println("Actor Path : " +  path);
      System.out.println("Actor Name : " + fileName);
      */ 
      FileInputStream is = new FileInputStream(path + "/" + fileName);
      Properties  p = new Properties();

      // Laden der Properties
      p.load(is);

      // Setzen der Properties
      this.description = p.getProperty("description").trim();
      this.color = ColorString.getColor(p.getProperty("color").trim());
      this.shortID = p.getProperty("id").trim();
      hasLight = Boolean.valueOf(p.getProperty("licht").trim()).booleanValue();

    } catch (IOException ioe) {
      ioe.printStackTrace();
    }   // catch
   } 

  /**
   * Liefert true, wenn es ein Lichtaktor ist
   * @return true wenn es ein Aktor mit Licht ist, andernfalls false
   */
  public boolean getHasLight() {
    return hasLight;
  } 

  /**
   * Sich selber speichern. 
   *
   * @param oos OutputStram
   *
   * @throws IOException Im Fehlerfall wird die Exception geworfen
   */
  public void saveAnimationObject(ObjectOutputStream oos) throws IOException {
    oos.writeObject(this.getClass());
    oos.writeDouble(originX);
    oos.writeDouble(originY);
    oos.writeUTF(fileName);
  } 


  /**
   * Einlesen der serailisierten Objekte dieses Objektes.
   *
   * @param ois InputStream
   * @param path Pfad wo sich die Beschreibungsdatei befindet
   *
   * @throws IOException Diese wird geworfen
   */
  public AnimationObject loadAnimationObject(ObjectInputStream ois, 
                                  String path) throws IOException {
    setBaseValues((int) ois.readDouble(), (int) ois.readDouble(), 
                  (String) ois.readUTF(), path);
    return this;              
  } 

  /**
   * Klonen dieses Objektes und Rueckgabe der Referenz auf dieses neue Objekt.
   *
   * @return Das erzeugte Objekt
   */
  public AnimationObject cloneObject() throws Exception {
    return (AnimationObject) (ObjectCloner.deepCopy(this));
  } 

  /**
   * Zeichnet den Actor
   * 
   * @param gr Grafikkontext
   * 
   * @see wesen.robot.AnimationObject#paint(Graphics)
   */
  public void paint(Graphics gr) {
    gr.setColor(color);

    // Zeichnet den Aktor
    gr.fillOval(getDrawX(), getDrawY(), diameter, diameter);
    gr.setColor(Color.red);
    gr.setFont(DRAW_FONT);
    gr.drawString(shortID, (int) (x - diameter / 2 + 6), 
                  (int) (y + 6));
    super.drawMark(gr);
  }   // paint
 
}     // Actor

