package wesen.robot;

import java.util.*;
import java.awt.*;
import wesen.util.*;
import java.io.*;
import wesen.simulator.*;

/**
 * Die Basisklasse von Robotern und Aktoren
 * 
 * @author Wesen-Team
 * @version 1.0
 * @since 04.07.2001
 */
abstract public class AnimationObject implements Serializable {

  /**
   * Ueberschreiben der serialisierungs-ID, damit werden InvalidClassException ausgeschlossen.
   * Besser waere alle nicht fuer die Serialsierung benutzten Variablen als transient zu
   * deklarieren, dann wuerde bei einer Aenderung der nicht relevanten Variablen dieses
   * Problem nicht auftreten.
   */
  static final long               serialVersionUID = 5229886082550730622L;

  /**
   * Font welche zur Beschriftung der Roboter bzw. Aktoren gebraucht wird
   */
  protected final static Font     DRAW_FONT = new Font("Helvetica", Font.PLAIN, 
          14);

  /**
   * Originiale Position des Objektmittelpunktes in der X-Achse, wird z.B.
   * fuer Reset benutzt
   */
  protected double                originX;

  /**
   * Name des Attributes welches geandert hat
   */
  protected String                changedName;

  /**
   * Referenz auf das geanderte Attribute
   */
  protected Object                changedValue;

  /**
   * Originale Position des Objektmittelpunktes in der Y-Achse, wird z.B fuer Reset benutzt
   */
  protected double                originY;

  /**
   * Aktule Position des Objektmittelpunktes in der X-Achse, ist eigentlich
   * nur fuer bewegte Objekte. Zur Vereinfachung ist es aber in dieser Klasse.
   */
  protected double                x;

  /**
   * Aktule Position des Objektmittelpunktes in der Y-Achse, ist eigentlich nur fuer bewegte Objekte
   * Zur Vereinfachung ist es aber in dieser Klasse.
   */
  protected double                y;

  /**
   * Volltextbeschreibung des Objektes, wird z.B. als ToolTipText benutzt.
   */
  public String                   description;

  /**
   * Kurzidentifikation z.B L=Licht (L1 .. LX), S=Stein, B=Blauer Roboter
   */
  public String                   shortID;

  /**
   * Referenz auf den moeglichen BeanEditor
   */
  protected transient BeanEditor    beanEditor;

  /**
   * Filename (Prefix und Suffix) z.B. stein.propeties
   */
  String                          fileName;

  /**
   * Hauptfarbe des Objektes
   */
  public Color                    color;

  /**
   * Ist das Objekt markiert, z.B. um es zu loeschen
   */
  protected boolean               isMarkOn = false;

  /**
   * Durchmesser des Objektes
   */
  protected int                   diameter;

  /**
   * Falls eine Beaneditor auf dieses Objekt sichtbar ist, so muss diesem mitgeteilt
   * falls das Objekt einen Wert aendert
   */
  private transient NewObservable observable;

  /**
   * Die Implementierung muss sich selber zeichnen koennen.
   * 
   * @param gr Grafikkontext
   * 
   * @see java.awt.Component#paint(Graphics)
   */
  abstract public void paint(Graphics gr);

  /**
   * Die Implementierung muss sich und seine Objekte selber speichern koennen.
   * Hier wird alles gespeichert welches nicht aus einer Property-Datei oder XML-Datei
   * kommt. Normalerweise sind dies die Objekte welche durch Benutzeraktionen direkt am
   * Objekt geaendert werden koennen wie z.B. die Position in der Simulationsumgebung.
   * 
   * @param oos Der OutputStream
   * 
   * @throws IOException Fehler muessen geworfen werden.
   * 
   * @see wesen.simulator.AnimationVector#saveWorld(ObjectOutputStream)
   */
  abstract public void saveAnimationObject(ObjectOutputStream oos) 
          throws IOException;

  /**
   * Die Implementierung muss sich und seine Objekte selber lesen konnen.
   * 
   * @param ois InputStream
   * @param path Dateiverzeichnis wo sich die XML- und Propertie-Dateien befinden.
   * 
   * @throws IOException
   * 
   * @see wesen.simulator.AnimationVector#readWorld(ObjectInputStream)
   */
  abstract public AnimationObject loadAnimationObject(ObjectInputStream ois, 
          String path) throws IOException;

  /**
   * Fuer das Clipboard wird das Objekt geklont (deep copy).
   * 
   * @return Das neue Objekt
   * 
   * @throws Exception Werfen des Fehlers
   * 
   * @see wesen.util.ObjectCloner
   */
  abstract public AnimationObject cloneObject() throws Exception;

  /**
   * Rueckgabe ob ein Punkt von diesem Objekt belegt ist.
   * 
   * @param iX Koordinate in der X-Achse
   * @param iY Koordinate in der Y-Achse
   * 
   * @return true, falls der Punkt von diesem Objekt belegt ist, andernfalls false
   */
  public boolean pointIsPart(int iX, int iY) {
    return (Geometrie.getDistance((int) x, (int) y, iX, iY) <= diameter / 2);
  } 

  /**
   * Methode welche abhandelt falls zwei Objekte zusammentreffen.
   * Fuer bewegt Objekte muss sicherlich eine Implementierung erfolgen.
   * 
   * @param b Das Objekt mit welchem der Zusammenstoss passiert ist
   */
  public void hit(AnimationObject b) {}

  /**
   * Rueckgabe der aktuellen Position in der X-Achse
   * 
   * @return Rueckgabe der Position in der X-Achse
   */
  public double getX() {
    return x;
  } 

  /**
   * Setzen von X, d.h. das Zentrum des Objektes in der X-Achse
   *
   * @param x X
   */
  public void setX(double x) {
    this.x = x;
  } 

  /**
   * Einen neuen Observer fuer dieses Objekt registrieren.
   *
   * @param observer Name des Observers
   */
  public void registerObserver(Observer observer) {
    observable = new NewObservable();

    observable.addObserver(observer);
  } 

  /**
   * Alle Observer fuer dieses Objekt entfernen.
   */
  public void unRegisterObserver() {
    observable.deleteObservers();
  } 

  /**
   * Rueckgabe der aktuellen Position in der Y-Achse
   * 
   * @return Rueckgabe der Position in der Y-Achse
   */
  public double getY() {
    return y;
  } 

  /**
   * Setzen von y, d.h. das Zentrum dieses Objektes
   *
   * @param y Y
   */
  public void setY(double y) {
    this.y = y;
  } 

  /**
   * Setzen ob das Objekt markiert ist, wird gebraucht damit sich das Objekt selber
   * korrekt zeichnen kann.
   * 
   * @param isMarkOn
   * 
   * @todo Zurzeit kann nur ein Objekt zur gleichen Zeit markiert sein. Andernfalls
   * ist dies ein Designfehler.
   */
  public void setMark(boolean isMarkOn) {
    this.isMarkOn = isMarkOn;
  } 

  /**
   * Rueckgabe des Objektesdurchmesser in Punkten
   * 
   * @return Objektdurchmesser in Punkten
   */
  public int getDiameter() {
    return diameter;
  } 

  /**
   * Setzen der Ursprungsposition in der Umgebung, normalerweise wirde diese durch
   * Benutzerinteraktionen gesetzt.
   * 
   * @param x Position in der X-Achse
   * @param y Position in der Y-Achse
   */
  public void setPos(int x, int y) {
    this.x = originX = x;
    this.y = originY = y;
  } 

  /**
   * Rueckgabe der oberen, linken Position des Roboters in der X-Achse
   * Diese Koordinate wird zum Zeichnen benoetigt
   * 
   * @return Obere linke Ecke in der X-Achse
   */
  public int getDrawX() {
    return (int) (x - diameter / 2);
  } 

  /**
   * Rueckgabe der oberen, linken Position des Roboters in der Y-Achse
   * Diese Koordinate wird zum Zeichnen benoetigt.
   * 
   * @return Obere linke Ecke in der Y-Achse
   */
  public int getDrawY() {
    return (int) (y - diameter / 2);
  } 

  /**
   * Rueckgabe der kurzen Objektidentifikation
   * 
   * @return Rueckgabe der Objektidentifikation
   */
  public String getShortID() {
    return shortID;
  } 

  /**
   * Setzen der kurzen Objektidentifikation
   * 
   * @param shortID Identifikationsstring
   * @see wesen.robot.ObjectVector#setFreeIDNumber(AnimationObject)
   */
  public void setShortID(String shortID) {
    this.shortID = shortID;
  } 

  /**
   * Setzen der Objekte auf die Ursprungsposition, normalerweise wurde die
   * Originalposition vom Benutzer gesetzt. Diese Funktion ist nur fuer bewegte
   * Objekte von Relevanz.
   */
  public void resetPosition() {
    x = originX;
    y = originY;
  } 

  /**
   * Rueckgabe des ToolTipTextes dieses Objektes
   *
   * @return Tooltiptext
   */
  public String getToolTipText() {
    return shortID + ": " + description;
  } 

  /**
   * Hindergrundfarbe fuer dieses Objekt setzen.
   *
   * @param strColor Die Farbe in der Form eines String, z.B. red fuer 
   * Color.red
   */
  public void setColor(String strColor) {
    color = ColorString.getColor(strColor);
  } 

  /**
   * Rueckgabe der Hintergrundfarbe dieses Objektes als String.
   * Z.b Color.red gibt red zurueck.
   *
   * @return Farbe als String
   */
  public String getColor() {
    return ColorString.getColorString(color);
  } 

  /**
   * Diese Objekt wird entfernt, dabei muss sichergestellt werden, dass
   * der Beaneditor geschlossen wird und eventuelle Observer abgemeldet 
   * werden.
   */
  protected void removeObject() {
    if (beanEditor != null && beanEditor.isVisible()) {
      observable.deleteObservers();
      beanEditor.setVisible(false);
      beanEditor.dispose();

      beanEditor = null;
      observable = null;
    } 
  } 

  /**
   * Aktiviert den Beaneditor fuer diese Objekt.
   *
   * @param umgebung Referenz auf die Simulationswelt
   */
  public void registerBeanEditor(Umgebung umgebung) {
    if (beanEditor == null ||!beanEditor.isVisible()) {
      removeObject();

      beanEditor = new BeanEditor(this, umgebung, this.getToolTipText());
    } 

    // Der Beaneditor bekommt den Focus
    beanEditor.requestFocus();
  } 

  /**
   * Wird von einem Observer abgefrag, falls er eine Melduung ueber die
   * Anederung eines Attribues erhaelt.
   *
   * @return Name des Attributes welches geandert wurde.
   */
  public String getPropertyName() {
    return changedName;
  } 

  /**
   * Wird von einem Observer abgefragt, falls er eine Meldlung ueber die 
   * Anederung eines Atttributes erhaelt.
   *
   * @return Referenz auf das Objekt welches sich geandert hat.
   */
  public Object getNewValue() {
    return changedValue;
  } 

  /**
   * Zeichnet die Markierung eines Objektes.
   * 
   * @param gr Krafikkontext
   */
  protected void drawMark(Graphics gr) {
    if (observable != null) {
      changedName = "y";
      changedValue = new Double(y);

      observable.notifyObservers(this);

      changedName = "x";
      changedValue = new Double(x);

      observable.notifyObservers(this);
    } 

    if (isMarkOn) {

      // Zeichnen eines schwarzen Rechteckes, dessen Ecken selber kleine Rechtecke
      // enthaelt
      gr.setColor(Color.black);
      gr.drawRect((int) x - diameter / 2, (int) y - diameter / 2, diameter, 
                  diameter);
      gr.drawRect((int) x - diameter / 2 - 2, (int) y - diameter / 2 - 2, 4, 4);
      gr.drawRect((int) x + diameter / 2 - 2, (int) y - diameter / 2 - 2, 4, 4);
      gr.drawRect((int) x - diameter / 2 - 2, (int) y + diameter / 2 - 2, 4, 4);
      gr.drawRect((int) x + diameter / 2 - 2, (int) y + diameter / 2 - 2, 4, 4);
    }   // if
   }    // drawMark
 
}

