package wesen.robot;

import java.awt.*;
import java.io.*;
import java.util.*;

import wesen.simulator.*;
import wesen.util.*;

/**
 * Der Sensor gehrt zu einem Roboter und ist dort links, bzw. rechts Vorne
 * plaziert. Er holt sich die Werte der Aktoren selber ber den AnimationVector.
 *
 *
 * @author Wesen-Team
 * @version 1.0
 * @since 04.07.2001
 */
public class Sensor implements Serializable   {
  
  /**
   * Sichtwinkel Sensor.
   */
  double                viewAngle = Math.PI / 6;
  // PI/2 =>90, wurde auf 30 gendert
  //double                viewAngle = Math.PI / 2;
  
  
  /** 
   * Reichweite des Sensors.
   * In diesem Bereich mssen Aktoren sein um den Sensor stimulieren zu knnen.   
   */
  public double range; 
  
  /** Der aktuelle Sensorwert (ist zwischen 0 und 1) */
  protected double value;
  
  /** Der letzte Senesorwert (dient zum Tendenz berechnen) */
  protected double lastValue;    
  
  /**
   * Definiert den Motorfaktorbereich
   * Der Sensorwert wird dabei mit motorFaktorX
   * auf 1.0 +/- Sensitivity gemappt.
   *
   * Bsp. Sensitivity 0.1:
   * - Bremsung ist zwischen 0.9 und 1.0
   * - Beschleunigung ist zwischen 1.0 und 1.1
   *
   * Bsp. Sensorwert 0.6 positiv einwirkend: 
   * - ergibt Faktor 1.06
   */
  protected double sensitivity = 0.1;
  
  /**
   * X Position des Sensors.
   * Muss nach jedem Schritt berechnet werden
   */ 
  double posX;            
   
  /**
   * Y Position des Sensors.
   * Muss nach jedem Schritt berechnet werden
   */
  double posY;
  
  /**
   * Gibt an, ob es sich um den rechten oder linken Sensor handelt.
   * True - rechter Sensor, false - linker Sensor.
   */
  public boolean blnIsRight;     
  
  /** 
   *  Gibt an, wie der linke Motor des Roboters stimuliert werden soll
   *
   *   '-'  ist bremsen 
   *   '\0' ist keine Bewegung
   *   '+'  ist Beschleunigen
   *   't'  ist Trend, d.h. '-', '\0' oder '+' je nach Sensorwertvernderung      
   */
  char motorFactorLeft='\0'; //'\0' 
  
  /**
   * Ist Analog zu motorFactorLeft
   * @see Sensor#motorFactorLeft
   */
  char motorFactorRight='\0';
    
  /** Dieser Sensor gehrt zum Roboter rob 
   * @supplierCardinality 1
   * @supplierRole hat*/  
  Roboter rob;            
    
  /** Konstruktor fuer XML */    
  public Sensor() {}  
  
  /**
   * Dieser Konstruktor wird nur fr Testzwecke gebraucht
   * 
   * @param Gleich wie bei den gleichnamigen Instanzvariablen.
   */
  public Sensor(boolean blnIsRight, double range, Roboter rob, char motorFactorLeft, char motorFactorRight){
    this.blnIsRight = blnIsRight;
    this.range = range;
    this.rob = rob;
    this.motorFactorLeft=motorFactorLeft;
    this.motorFactorRight=motorFactorRight;
    calcNewPos();
  }    
      
  /** 
   * Berechne die neue Position des Sensors<br>      
   * motorSpace - Abstand der Motoren (und auch der Sensoren)<br>
   * angle - Richtungswinkel des Roboters (in Radian)<br>
   * posRobMX, posRobMY - Positionen des Mittelpunktes des Roboters<br>
   *
   * Info: gibts mehrere Sensoren auf einer Seite, so wird die Methode 
   * calcNewPosForMultipleSensors verwendet.
   */
  public void calcNewPos(){
    double delta = 0;
    if (blnIsRight)
      delta = -Math.PI/4;
    else
      delta = Math.PI/4;
    
    posX=rob.getX() + Math.sqrt(2)*rob.motorspace/2*Math.cos(rob.alpha + delta);
    posY=rob.getY() - Math.sqrt(2)*rob.motorspace/2*Math.sin(rob.alpha + delta);
  }
 
  /**
   * Setze die neue Position des Sensors
   */ 
  private void setNewPos(double x, double y){
    this.posX = x;
    this.posY = y;
  } // method
  
  /**
   * Setze neuen Wert und schneide ihn auf Intervall (0, 1) zu
   */
  public void setNewValue(double val){    
    //letzen Wert speichern
    lastValue = this.value; 
    
    //Neuen Wert anpassen
    if (val<0) val =0;
    else if( val>1) val = 1;
    else this.value = val;
  }
  
  /** 
   * Setze neue Motorgeschwindigkeiten
   * aufgrund mehrerer Kriterien
   */
  public void setNewMotorSpeeds(){    
    double v=0;
    
    //Passe Motorgeschwindigkeiten direkt an        
    v=rob.motorleft.v*getMappedValue(motorFactorLeft, sensitivity, value);   
    rob.motorleft.setV(v);
    
    v=rob.motorright.v*getMappedValue(motorFactorRight, sensitivity, value);
    rob.motorright.setV(v);
    
    // Debug
    if (rob.getDebugValueSpeedsLeftRightTrend() && Simulator.withDebug){
      System.out.println(rob.getToolTipText());
      System.out.println("Blickwinkel: " + Math.toDegrees(viewAngle)); 
      if (blnIsRight)
        System.out.println("Rechter Sensor: " + this.value);
      else
        System.out.println("Linker Sensor: " + this.value);
      System.out.println("Speed Left: " + rob.motorleft.v + ", MFleftTrend=" + trend2Factor(motorFactorLeft));
      System.out.println("Speed Right: " + rob.motorright.v + ", MFrightTrend=" + trend2Factor(motorFactorLeft));
      System.out.println();
    }
    
  }
  
  /**
   * Liefert den Factor aufgrund des Trends von lastValue -> Value
   * D.h. wenn factor = 't' ist. 
   *    
   * @param  factor
   * @return factor wenn kein Trend vorliegt
   */
  private char trend2Factor(char factor){
    if (factor=='t'){
      if (lastValue<value) return '+';
      else if (lastValue>value) return '-';
      else return '\0';
    }
    return factor;
  }
  
  /** 
   * Macht Mapping (Sensorwert -> Motorgeschwindigkeit)
   * und liefert dabei die Motorgeschwindigkeit.<br>
   * 
   * Todo: Parameter durch Instanzvariablen ersetzen
   */
  private double getMappedValue(char factor, double sensitivity, double value){
    
    //Wenn Tendenz, Ordne ihr eine Proportionalitt zu
    factor = trend2Factor(factor);
    
    //Werte Proportionalitt
    if (factor=='+'){
      //if (test) System.out.println("factor=='+'");
      return value*sensitivity+1;
    }
    else if (factor =='-'){
      //if (test) System.out.println("factor=='-'");
      return 1-value*sensitivity;
    }
    else {
      //if (test) System.out.println("else");
      return 1;    
    }
  }
  
  
 
  /**
   * Abstrakte Methode (muss implementiert werden)
   * ov    - Enthlt Roboter und andere Aktoren
   * index - Position von this.rob innerhalb des ov
   */
  // abstract void updateSensorValue(ObjectVector ov, int index);
  // void updateSensorValue(ObjectVector ov, int index){};
  
  /****************************************************************************************
   * Klassenmethoden
   ***************************************************************************************/
  
  /**
   * Berechne die neue Position von allen Sensoren einer Seite
   * @param sensors (alle linken, bzw. alle rechten Sensoren eines Roboters)
   * @param rechte und linke Sensoren knnen im Vector sensors gemischt 
   * @param werden
   */
  public static void calcNewPosForMultipleSensors(Vector sensors){
    
    Sensor firstLeft = null, firstRight = null, sensor = null;
    
    for (int i = 0; i < sensors.size(); i++) {
      
      sensor = (Sensor)sensors.elementAt(i);

      // Neue Position nur beim ersten linken oder rechten Sensor berechnen und merken
      // Weitere Position kopieren
      if (sensor.blnIsRight && firstRight == null){
        ((Sensor) sensors.elementAt(i)).calcNewPos();
        firstRight = (Sensor)sensors.elementAt(i); //Ersten rechten merken        
      }else if (!sensor.blnIsRight && firstLeft == null){
        ((Sensor) sensors.elementAt(i)).calcNewPos();
        firstLeft = (Sensor)sensors.elementAt(i); //Ersten linken merken        
      }else if (sensor.blnIsRight){
        sensor.setNewPos(firstRight.posX, firstRight.posY);  
      }else if (!sensor.blnIsRight){
        sensor.setNewPos(firstLeft.posX, firstLeft.posY);  
      }
      
    } // for
    
    
    
  } //method
  
  /** Selbsterklrend */
  public void setRoboter(Roboter roboter) {
        this.rob = roboter;
  }    
  
  /** Selbsterklrend */
  public void setPosition(String cPos) {
      blnIsRight = (cPos.equals("R"))? true: false;
  }
  
  /** 
   * Liefert die Sensor-'Position'
   * @return Wenn linker Sensor -> "L"
   * @return Wenn rechter Sensor -> "R"
   */
  public String getPosition() {
      return (blnIsRight)? "R": "L";
  }

  /** Selbsterklrend */
  public void setMotorFactorLeft(String strFactor) {
    motorFactorLeft=(strFactor != null && strFactor.length() > 0)? strFactor.charAt(0): '\0';
  }  
 
  /** Selbsterklrend */
  public String getMotorFactorLeft() {
    return String.valueOf(motorFactorRight);
  }  

  /** Selbsterklrend */
  public void setMotorFactorRight(String strFactor) {
    motorFactorRight=(strFactor != null && strFactor.length() > 0 )? strFactor.charAt(0): '\0';
  }  
  
  /** Selbsterklrend */
  public String getMotorFactorRight() {
    return String.valueOf(motorFactorRight);
  }  
  
  /** Selbsterklrend */
  public void setViewAngle(double gradViewAngle) {
     viewAngle = Math.toRadians(gradViewAngle); 
  }  
  
  /** Selbsterklrend */
  public double getViewAngle() {
     return Math.toDegrees(viewAngle); 
  } 
  
    
}