package wesen.simulator;

import java.awt.*;
import com.symantec.itools.javax.swing.JMaskedTextField;
import java.io.*;
import java.util.Observer;

/**
 * Eine Animation muss die folgende Methoden implementieren, damit sie mit der
 * Simulation kommunizieren kann. Damit wird der Simulator unabhaengig von der
 * Implementierung der Animation. Der Simulator kommuniziert nur mit dem Objekt
 * welche dieses Interface implementiert.
 * 
 * @author Wesen-Team
 * @version 1.0
 */
public interface AnimationVector {

  /**
   * Hinzufuegen eines neuen Objektes in die Simulationswelt.
   * 
   * @param x Position in der X-Achse
   * @param y Position in der Y-Achse
   * @param fileName Name der Datei
   */
  public void addObject(int x, int y, String fileName);

  /**
   * Die Objekte muessen sich selbst in der entsprechenden Umgebung zeichnen
   * koennen.
   * 
   * @param g2 Grafikkontext
   */
  public void drawIt(Graphics2D g2);

  /**
   * Verzeichnis wo sich die XML bzw. Propertie-Dateien befinden, welche die Objekte
   * beschreiben.
   * 
   * @return Relativer Pfad
   */
  public String getDirOfRobotData();

  /**
   * Es koennen zwei verschiedene Klassenbeschreibungen existieren. Z.B. Xml
   * und Properties. Dateinamen mit dieser Dateiendung werden zuerst im Popupmenu
   * ausgegeben.
   * 
   * @return Dateinamenserweiterung
   */
  public String getRobotSuffix();

  /**
   * Es koennen zwei verschiedene Klassenbeschreibungen existieren. Z.B. Xml
   * und Properties. Dateinamen mit dieser Dateiendung werden als zweite im Popupmenu
   * ausgegeben.
   * 
   * @return Dateinamenserweiterung
   */
  public String getActorSuffix();

  /**
   * Rueckgabe des Objekte welchen den durch X und Y bestimmten Punkt belegt.
   * 
   * @param iX Position in der X-Achse
   * @param iY Position in der Y-Achse
   * 
   * @return Index: Bei -1 der Punkt ist nicht belegt, >0 Das ensprechende Objekt.
   */
  public int getObjectAtPointID(int iX, int iY);

  /**
   * Rueckgabe des Objekt-Tooltipes welchen den durch X und Y bestimmten Punkt belegt.
   * 
   * @param iX Position in der X-Achse
   * @param iY Position in der Y-Achse
   * 
   * @return Index: Null fuer kein ToolTip andernfalls der ensprechende String.
   */
  public String getToolTipOfObjectAtPoint(int iX, int iY);

  /**
   * Setzen des Objekte an die durch X und Y bestimmte Position.
   * Wird z.B. benutzt wenn das Objekte mit der Maus neu platziert wird.
   * 
   * @param workId Der Index des beteiligten Objektes
   * @param iX Position in der X-Achse
   * @param iY Position in der Y-Achse
   */
  public void setPos(int workId, int iX, int iY);

  /**
   * Markierung fuer das entsprechende Objekte setzen bzw. aufheben.
   * Die Kopier- oder Loeschfunktion funktioniert nur auf markierten Objekten.
   * Die visuelle Darstellung muss von dieser Implementierung vorgenommen werden.
   * Auch muss eine Markierung von mehreren Objekten von dieser Implementierung
   * gehandhabt werden.
   * 
   * @param simulator Referenz auf den Simulator
   * @param workId Der Index des beteiligten Objektes
   * @param isMarkOn Ist die Markierung einzuschalten.
   */
  public void setMark(Simulator simulator, int workId, boolean isMarkOn);

  /**
   * Rueckgabe des Durchmessers des Objektes. Das Objekt kann nur rund bzw.
   * quatratisch sein.
   * 
   * @param workId Der Index des entsprechenden Objekt
   * 
   * @return Die Ausdehnung des Objektes.
   */
  public int getObjectDiameter(int workId);

  /**
   * Rueckgabe des markierten Objektes.
   * 
   * @return Index des markierten (>0) Objektes.
   */
  public int getMark();

  /**
   * Rueckgabe des Punktes der am weitesten von der Koordinate (0,0)
   * durch ein Objekte belegt ist. Die Simulationsanzeige wird noetigenfalls mit dem
   * Simulieren auf diese Groesse erweitert.
   * 
   * @return X und Y als Dimensions-Objekt
   */
  public Dimension getMaxOccupiedYX();

  /**
   * Beim ersten Simulationscyklus wird diese Methode aufgerufen. Danach wird nur
   * noch moveRoboter aufgerufen.
   * 
   * @see moveRoboter()
   */
  public void initRoboter();

  /**
   * Beim zweiten und jedem folgenden Simulationscyklus wird diese Methode aufgerufen.
   * Dabei sollten die animierten Objekte ihre neue Position berechnen.
   * 
   * @see initRoboter()
   */
  public void moveRoboter();

  /**
   * Objekte werden aus dem Clipboard in die Umgebung eingefuegt, dabei muss
   * das Einfuegen von dieser Implementation gehandhabt werden.
   * 
   * 
   * @param simulator Referenz auf den Simulator
   * @param umgebung Referenz auf die Simulationswelt
   * 
   * @return true - wenn das Einfuegen erfolgreich war, andernfalls false
   */
  public boolean getClipboard(Simulator simulator, Umgebung umgebung);

  /**
   * Die animierten Objekte besitzen einen Winkel. Den Bewegungswinkel. Dieser
   * Winkel kann durch den Benutzer verandert werden.
   * 
   * @param dGrad Den Winkel in Grad
   * @param markId Index des betroffenen Objektes
   */
  public void setAlphaGrad(double dGrad, int markId);

  /**
   * Setzt das markierte Objekt in das Clipboard.
   * 
   * @return true falls die Aktion erfolgreich war, andernfalls false
   * 
   * @see getClipboard()
   */
  public boolean setClipboard();

  /**
   * Enfernen eines einzelnen Objektes aus der Simualtionswelt.
   * Die Handhabung der Markierung muss entsprechend abgehandelt werden.
   * 
   * @param simulator Referenz auf den Simulator
   * @param workId Der Index des zu entfernenden Objektes
   */
  public void removeOneElement(Simulator simulator, int workId);

  /**
   * Die Welt einlesen. Die Objekte muessen sich selber wiederherstellen koennen.
   * Die Aktion wir vor dem Starten des Simulators durchgefuehrt, wenn dies der Benutzer
   * akzeptiert. Die Objekte sind serialisiert abgespeichert.
   * @param ois InputStream
   * 
   * @throws IOException Im Fehlerfalls muss eine IOException geworfen werden.
   * Dabei wird vom Simulator eine Fehlermedlung ausgegeben.
   * @see saveWorld(ObjectOutputStream)
   */
  public void readWorld(ObjectInputStream ois) throws IOException;

  /**
   * Der Benutzer waehlte eine neue Welt erstellen. Daher muessen alle
   * Objekte entfernt werden.
   * 
   * @param simulator Referenz auf den Simulator
   * @param umgebung Referenz auf die Umgebung
   */
  public void removeAllElements(Simulator simulator);

  /**
   * Die Objekte muessen sich selber speichern koennen.
   * Die Aktion wird beim Beenden des Simulators durchgefuerht, wenn dies der Benutzer
   * akzeptiert.
   * 
   * @param oos Das OutputStream
   * @see readWorld(ObjectInputStream)
   */
  public void saveWorld(ObjectOutputStream oos) throws IOException;

  /**
   * Die Objekte muessen in die Ausgangstellung gefahren werden. Wird vom Benutzer
   * ausgeloesst.
   */
  public void resetPosition();

  /**
   * Registrieren eines Observers.
   * Dieser muss informiert werden falls sich die Anzahl der Roboter in der Umgebung
   * verndert hat.
   * 
   * @param observer Der zu registrierenden Observer
   */
  public void registerObserver(Observer observer);

  /**
   * Rueckgabe der Anzahl Roboter in der Umgebung.
   * 
   * @return Anzahl Roboter
   */
  public int countNumberOfRoboters();

  /**
   * Started den Beaneditor falls der Punkt(iX, iY) durch ein entsprechendes
   * Objekt besetzt ist.
   * 
   * 
   * @param iX Position in der X-Achse
   * @param iY Position in der Y-Achse
   * @param umgebung Referenz auf die Simulationswelt
   */
  public void startBeanEditor(int iX, int iY, Umgebung umgebung);
}

