package wesen.simulator;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.beans.*;
import java.lang.reflect.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import wesen.robot.*;

/**
 * Dies ist ein primitiver Beaneditor welcher es erlaubt dynamisch Eigabenschaften eines animations 
 * Objektes zu ermitteln und diese zu editieren. 
 * Die editierbaren Eigenschaften werden anhand der setter- bzw. getter Methoden ermittelt.
 * Nur wenn beide Methoden vorhanden sind, wird die Eigenschaft im Editor aufgefuehrt.
 *
 * @author Hugo Graf
 * @version 1.0
 */
public class BeanEditor extends JFrame implements Observer {
  protected AnimationObject   m_bean;
  protected JTable            m_table;
  protected PropertyTableData m_data;

  /**
   * @supplierCardinality 1 
   */
  protected Umgebung          umgebung;
  protected int               validProp = 0;

  /**
   * Startet einen Editor fuer das bean Object.
   *
   * @param bean Animations Objekt fuer welchen der Editor gestartet werden soll.
   * @param umgebung Die View wo sich das Objekt befindet
   * @param title Eine verstaendliche Beschreibung welche den Benutzer erlaubt das
   * Objekt zu erkennen
   */
  public BeanEditor(AnimationObject bean, Umgebung umgebung, String title) {
    m_bean = bean;
    this.umgebung = umgebung;

    addWindowListener(new WindowClosingAdapter());

    m_data = new PropertyTableData(m_bean);
    m_table = new JTable(m_data);
    m_table.setRowSelectionInterval(0, 0);
    m_table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    
    m_table.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_ALL_COLUMNS);

    JScrollPane ps = new JScrollPane();

    ps.getViewport().add(m_table);
    getContentPane().add(ps, BorderLayout.CENTER);
    setDefaultCloseOperation(HIDE_ON_CLOSE);
    setTitle("Edit: " + title);
    setSize(300, 200);
    setVisible(true);
    setDefaultCloseOperation(javax.swing.JFrame.DO_NOTHING_ON_CLOSE);
    m_bean.registerObserver(this);
  }

  public void setObject(AnimationObject bean) {
    m_bean.unRegisterObserver();
    m_bean = bean;
    m_bean.registerObserver(this);
  }  

  /**
   * Wird ein Wert extern geandert, so muss der Editor diesen neuen Wert 
   * beim Objekt abholen. 
   *
   * @param obs Wird nicht gebraucht
   * @param obj Objekt welches seine Aenderung mitgeteilt hat.
   */
  public void update(Observable obs, Object obj) {
    if (obj == m_bean) {
      m_data.setProperty(((AnimationObject) obj).getPropertyName(), 
                         ((AnimationObject) obj).getNewValue());
    }   // if
   } 

  /**
   * Das Model der JTable. 
   *
   *
   * @author Hugo Graf
   */
  class PropertyTableData extends AbstractTableModel {
    protected String[][]  m_properties;
    protected int         m_numProps = 0;
    protected Vector      m_v;

    /**
     * Hiermit wird das Model erstellt und mit den Eigenschaften des Objektes abgefuellt.
     *
     *
     * @param bean Das animations Objekt fuer welches das Model angelegt werden soll.
     */
    public PropertyTableData(AnimationObject bean) {
      try {
        BeanInfo              info = 
          Introspector.getBeanInfo(m_bean.getClass());
        BeanDescriptor        descr = info.getBeanDescriptor();
        PropertyDescriptor[]  props = info.getPropertyDescriptors();

        m_numProps = props.length;
        m_v = new Vector(m_numProps);

        for (int k = 0; k < m_numProps; k++) {
          String  name = props[k].getDisplayName();
          boolean added = false;

          for (int i = 0; i < m_v.size(); i++) {
            String  str = 
              ((PropertyDescriptor) m_v.elementAt(i)).getDisplayName();

            if (name.compareToIgnoreCase(str) < 0) {
              m_v.insertElementAt(props[k], i);

              added = true;

              break;
            } // if
          } // for

          if (!added) {
            m_v.addElement(props[k]);
          } // if
        }   // for
 
        m_properties = new String[m_numProps][2];

        for (int k = 0; k < m_numProps; k++) {
          PropertyDescriptor  prop = 
            (PropertyDescriptor) m_v.elementAt(validProp);
          Method              mRead = prop.getReadMethod();
          Method              mWrite = prop.getWriteMethod();

          if (mRead != null && mRead.getParameterTypes().length == 0 
                  && mWrite != null) {

            // Es werden nur gueltige Objekte hinzugefuegt
            Object  value = mRead.invoke(m_bean, null);

            m_properties[validProp][0] = prop.getDisplayName();
            m_properties[validProp][1] = objToString(value);
            validProp++;
          } else {
            m_v.remove(validProp);
          } // else
        } // for
      } catch (Exception ex) {
        ex.printStackTrace();
        JOptionPane.showMessageDialog(BeanEditor.this, 
                                      "Error: " + ex.toString(), "Warning", 
                                      JOptionPane.WARNING_MESSAGE);
      } 
    }

    /**
     * Setz einen neuen Wert in das Model, dabei ist der name das Suchkriterium.
     *
     * @param name Name der Eigenschaft welche einen neuen Wert erhaelt
     * @param value Objekt welches den neuen Wert enthaelt
     */
    public void setProperty(String name, Object value) {
      for (int k = 0; k < m_numProps; k++) {
        if (name.equals(m_properties[k][0])) {
          m_properties[k][1] = objToString(value);

          m_table.tableChanged(new TableModelEvent(this, k));
          m_table.repaint();

          break;
        } // if
      } // for
    } 

    /**
     * Rueckgabe der anzahl Zeilen im Model.
     *
     * @return Anzahl Zeilen
     */
    public int getRowCount() {
      return validProp;
    } 

    /**
     * Rueckgabe der anzahl Spalten im Model.
     *
     * @return Anzahl Spalten
     *
     * @see
     */
    public int getColumnCount() {
      return 2;
    } 

    /**
     * Rueckgabe der Kopftexte
     *
     * @param nCol Spalte des gewuenschten Kofptextes
     *
     * @return Kopftext
     */
    public String getColumnName(int nCol) {
      return nCol == 0 ? "Eigenschaft" : "Wert";
    } 

    /**
     * Rueckgabe ob die Zelle editierbar ist? 
     * Es kann nur die Spalte 1 editiert werden, dies ist die Spalte mit den
     * Eigenschaftswerten.
     *
     * @param nRow Zeilenindex
     * @param nCol Spaltenindex
     *
     * @return Bei true die Zelle kann editiert werden, bei false nicht.
     */
    public boolean isCellEditable(int nRow, int nCol) {
      return (nCol == 1);
    } 

    /**
     * Rueckgabe des Wertes fuer die von der JTable gewnschte Zelle.
     *
     *
     * @param nRow Zeilenindex
     * @param nCol Spaltenindex
     *
     * @return Object 
     */
    public Object getValueAt(int nRow, int nCol) {
      if (nRow < 0 || nRow >= validProp) {
        return "";
      } 

      switch (nCol) {

        case 0:
          return m_properties[nRow][0];

        case 1:
          return m_properties[nRow][1];
      }

      return "";
    } 

    /**
     * Einen Wert von der JTable in das Model setzen. Dabei wird auch beim Objekt 
     * der neue Wert gesetzt.
     *
     * @param value Wert als Objekt 
     * @param nRow Zeilenindex
     * @param nCol Spaltenindex
     */
    public void setValueAt(Object value, int nRow, int nCol) {
      if (nRow < 0 || nRow >= validProp) {
        return;
      } 

      String              str = value.toString();
      PropertyDescriptor  prop = (PropertyDescriptor) m_v.elementAt(nRow);
      Class               cls = prop.getPropertyType();
      Object              obj = stringToObj(str, cls);

      if (obj == null) {
        return;  
      } 

      Method  mWrite = prop.getWriteMethod();

      if (mWrite == null || mWrite.getParameterTypes().length != 1) {
        return;
      } 

      try {
        // Es wird versucht den Eingabewert auf das Objekt abzubilden.
        mWrite.invoke(m_bean, new Object[] {
          obj
        });
        // Danach muss die Simulationsumgebung neu gezeichnet werden.
        umgebung.repaint();
      } catch (Exception ex) {
        ex.printStackTrace();
        JOptionPane.showMessageDialog(BeanEditor.this, 
                                      "Error: " + ex.toString(), "Warning", 
                                      JOptionPane.WARNING_MESSAGE);
      } 

      m_properties[nRow][1] = str;
    } 

    /**
     * Konvertiert eine durch value gegebenes Objekt in ein String Objekt und gibt dessen
     * Wert zurueckt.
     *
     * @param value Objekt welches konvertiert werden soll.
     *
     * @return Rueckgabe des Wertes als String
     */
    public String objToString(Object value) {
      if (value == null) {
        return "null";
      } 

      if (value instanceof Color) {
        Color col = (Color) value;

        return "" + col.getRed() + "," + col.getGreen() + "," + col.getBlue();
      } 


      return value.toString();       
      
    } 

    /**
     * Verwandelt den String str in die durch cls bestimmte Klasse.
     *
     * @param str String welche in die entsprechende Klasse verwandelt werden soll.
     * @param cls Klasse welche der String repraesentiert.
     *
     * @return Eine Referenz auf die erzeugte Klasse 
     */
    public Object stringToObj(String str, Class cls) {
      try {
        if (str == null) {
          return null;
        } 

        String  name = cls.getName();

        if (name.equals("java.lang.String")) {
          return str;
        } else if (name.equals("int")) {
          return new Integer(str);
        } else if (name.equals("long")) {
          return new Long(str);
        } else if (name.equals("float")) {
          return new Float(str);
        } else if (name.equals("double")) {
          return new Double(str);
        } else if (name.equals("boolean")) {
          return new Boolean(str);
        } 
        // Der Datentyp wird nicht unterstuetzt 
        return null;  
      } catch (Exception ex) {
        return null;
      } // catch
    } // stringToObj

  }

  /**
   * Schliessen des Beaneditor, dabei muss sichergestellt werden das der Observer entfernt wird
   *
   * @author Hugo Graf
   */
  class WindowClosingAdapter extends WindowAdapter {

    /**
     * Schliessen des Fensters und entferenen des  Observer.
     *
     * @param event Event
     *
     * @see java.awt.event.WindowAdapter#windowClosing(WindowEvent)
     */
    public void windowClosing(WindowEvent event) {
      event.getWindow().setVisible(false);
      event.getWindow().dispose();
      m_bean.unRegisterObserver();
    } 
  } // class WindowClosingAdapter

}

