package wesen.simulator;

import java.awt.Component;
import javax.swing.*;
import java.awt.Dimension;
import wesen.util.*;


/**
 * Ist zustndig fuer die animierte Simulation. Diese laeuft in einem 
 * eigenen Thread.
 * 
 * @author Wesen-Team
 * @version 1.0
 */
class Simulate implements Runnable {

  /**
   * Anzahl Bilder pro Sekunde
   */
  private final int       NUMBER_OF_FRAMES = 12;

  /** Referenz auf das Top JFrame 
   * @supplierCardinality 1*/
  private Simulator simulator;

  /**
   * Referenz auf die Umgebung
   * @supplierCardinality 1*/
  private Umgebung        umgebung;

  
  /**
   * 0 = kein Schrittmodus, >0 Schrittmodus mit der Anzahl Cyclen
   */
  private int             iStep;
  
  /**
   * Kann nur einmal instanciert werden daher Singleton Muster
   * @label Singleton
   */ 
  private static Simulate _instance = null;
  
  /**
   * Die Simulation ist am laufen oder nicht
   */
  private boolean         mayRunning = false;
  
  /**
   * Thread der Simulation
   */
  private Thread          simThread;

  /**
   * Erzeugt eine neue Instanz dieser Klasse.
   *
   * @param simulator Referenz auf den Simulator
   * @param umgebung Referenz auf Umgebung
   */
  protected Simulate(Simulator simulator, Umgebung umgebung) {
    this.simulator = simulator;
    this.umgebung = umgebung;
  }
  
  /**
   * Rckgabe einer Instanz diese Klasse. Muster Singelton
   *
   * @param simulator Referenz auf den Simulator
   * @param umgebung Referenz auf die Umgebung
   *
   * @return Referenz auf ein Objekt dieser Klasse.
   */
  public static Simulate getInstance(Simulator simulator, Umgebung umgebung) {
    if (_instance == null) {
      _instance = new Simulate(simulator, umgebung);
    } // if

    return (_instance);
  } // getInstance


  /**
   * Startet die Simulation
   *
   * @param iStep 0 = Kein Schrittmodus, >0 Schritmodus mit Anzahl der Cyclen
   */
  public void simStart(int iStep) {
    if (isSimRunning()) {
      simStop();
    }
    else  {
      this.iStep = iStep;
      simThread = new Thread(this);

      simThread.start();
    }  // else
  }  // simStart

  /**
   * Stoppen der Simulation
   */
  public void simStop() {
    if (isSimRunning()) {
      mayRunning = false;

      simThread.interrupt();

      simThread = null;
      simulator.stopSimulation();
    } // if
  } // simStop

  /**
   * Gibt zurck ob der Simualtionsthread am laufen ist
   *
   * @return true = Simulationsthread luft, andernfalls false
   */
  public boolean isSimRunning() {
    return simThread != null && simThread.isAlive();
  } // isSimRunning()

  

  /**
   * Bewegungs- und Anzeigeschleife der Simulation
   */
  public void run() {
    long          t = System.currentTimeMillis();
    int           delay = (int) (1000 / NUMBER_OF_FRAMES);
    AnimationVector av = simulator.getRoboterVector();
    int           cycles = 0;
    
    if(av.getMark() >= 0) {
      // Markierung muss vom Objekt entfernt werden
      av.setMark(simulator, -1, false);
      umgebung.forceRepaint();
    }   
    
    mayRunning = true;
    umgebung.setWorkId(-1);
    
    // Die Umgebung wird noetigenfalls auf das am weitesten entfernten Objekt vergroessert.
    Dimension dim = av.getMaxOccupiedYX();
    int height = Math.max((int) dim.getHeight(), umgebung.getHeight());
    int width  = Math.max((int) dim.getWidth(), umgebung.getWidth());
    
    if(height != umgebung.getHeight() || width != umgebung.getWidth() ) {
      umgebung.setPreferredSize(new Dimension(width, height));
      simulator.pack();
      // umgebung.revalidate();
      MessageHandler.showTextFieldMessage("frameresize", MessageHandler.NORMAL_COLOR);
    }  

    
    if(iStep == 0) {
      simulator.setResizable(false);
    }  

    while (mayRunning && (iStep == 0 || cycles < iStep)) {
      if(cycles == 0) {
        av.initRoboter();
      } else { 
        // Die Roboter werden in die Position fuer das naechste Bild gebracht
        av.moveRoboter();
      }  

      // Die Roboer werden neu gezeichnet und angezeigt
      umgebung.repaint();

      cycles++;

      try {

        // Damit wird erricht, dass die Anzeige der Bilder in einem
        // genauen Rhytmus erscheinen.
        t += delay;

        Thread.sleep(Math.max(0, t - System.currentTimeMillis()));
      }   // try
       catch (InterruptedException ie) {
        break;
      }   // catch
     }    // while
     if(!simulator.isResizable()) {
       simulator.setResizable(true);
     }   // if
   }      // run
}         // Simulate

