package wesen.util;

import java.util.StringTokenizer;
import java.util.*;
import javax.swing.*;
import java.awt.*;

/**
 * Diese Klasse ermoeglicht es Frage/Fehler/Meldungen in einem modalen Fenster zu zeigen. 
 * Diese Dialoge koennen Sprachunabhngig gehalten werden, da die Texte aus einem
 * Propertie-File gelesen werden. Zudem werden hier auch die Meldungen fuer die Statuszeile
 * gehandhabt.
 *
 * @author Hugo Graf
 * @version 1.0
 * @since 18.07.2001
 */
public class MessageHandler {
  
  /** Die Meldung fuer die Statuszeile - eine "normale" Meldung */  
  static final public Color NORMAL_COLOR = Color.black;
  
  /** Die Meldung fuer die Statuszeile - eine Fehlermeldung */
  static final public Color ERROR_COLOR = Color.black;
  
  /** Name der Propertiedatei, welche die Ressourcen enthalten */
  static String             _strPropName;
  /**  Welcher Sprachcode (Locale) */
  static Locale             _language;
  /** JTextFiled welches fuer die Ausgabe von Meldungen bestimmt ist */
  static JTextField         _messageField1;

  /**
   * Initialisierung dieses Objektes.
   *
   * @param strPropName Name der Propertie-Datei
   * @param language Sprachcode, z.B. Locale.GERMAN
   * @param messageField1 JTextField welches fuer die Ausgabe von einfachen Meldungen benutzt wird
   */
  static public void setResourceFile(String strPropName, Locale language, 
                                     JTextField messageField1) {
    _strPropName = strPropName;
    _language = language;
    _messageField1 = messageField1;
  } 

  /**
   * Mittels dem Schluessel (Key) wird die entsprechende Meldung aus der Propertie-Datei
   * ermittelt und zurck gegeben.
   *
   * @param key Schlssel fuer die Ermittlung des Textes
   *
   * @return Der gefundene Text in der Propertie-Datei
   */
  public static String getMessage(String key) {
    String  s = null;

    try {
      ResourceBundle  rb = ResourceBundle.getBundle(_strPropName, _language);

      s = rb.getString(key);
    } catch (MissingResourceException e) {
      s = null;
    } // catch

    return s;
  }  // getMessage

  /**
   * Erstellt aus einem String ein Objekt von Strings, dabei ist der "|" der entsprechende
   * Delimiter, welcher eine neue Zeile erzeugt.
   *
   * @param strMessage Meldungstext
   *
   * @return Objekte von Strings
   */
  static private Object[] getMessageObject(String strMessage) {

    
    StringTokenizer stMessage = new StringTokenizer(strMessage, "|");
    Object[]        strArrayMessages = new Object[stMessage.countTokens()];

    for (int i = 0; stMessage.hasMoreTokens(); i++) {
      strArrayMessages[i] = new String(stMessage.nextToken());
    }   // for
 
    return strArrayMessages;
  }   // getMessages
 
  /**
   * Ausgabe einer Meldung in ein JTextField der Applikation, z.B. Statuszeile.
   *
   * @param strMessageKey Schluessel fuer den Meldungstext
   * @param color Besimmt die Farbe des Meldungstextes (NORMAL_COLOR, ERROR_COLOR)
   */
  public static void showTextFieldMessage(String strMessageKey, Color color) {
    _messageField1.setForeground(color);
    _messageField1.setText(getMessage(strMessageKey));
  } // showTextFieldMessage


  /**
   * Ausgabe einer Meldung in einem seperaten Dialog.
   * Der Benutzer bekommt die in iOptionType angegebene Interaktionsmglichkeiten.
   *
   * @param comp Vaterkomponente (bestimmt die Platzierung des Dialoges)
   * @param strQuestionKey Schluessel fuer den Fragetext
   * @param strTitleKey Schluessel fuer den Titeltext
   * @param iOptionType Welche Art des Dialoges (YES_NO_CANCEL_OPTION usw.)
   * @param iMessageType Welche Art der Meldung (ERROR_MESSAGE usw.
   *
   * @return Die Wahl des Benutzers
   * @see java.lang.StringBuffer#append(boolean)
   * @see javax.swing.JOptionPane#showOptionDialog(Component, Object,  String, int, int, Icon, Object[], Object)
   */
  static public int askQuestion(Component comp, String strQuestionKey, 
                                String strTitleKey, int iOptionType, 
                                int iMessageType) {
    Object[]  strDialogWarningButtonText;


    Object[]  strArrayMessages = getMessageObject(getMessage(strQuestionKey));

    if (iOptionType == JOptionPane.YES_NO_OPTION) {
      strDialogWarningButtonText = new Object[2];
    } else {
      strDialogWarningButtonText = new Object[3];
    } 

    switch (iOptionType) {

      case JOptionPane.YES_NO_OPTION:

      case JOptionPane.YES_NO_CANCEL_OPTION:
        strDialogWarningButtonText[0] = getMessage("yes");
        strDialogWarningButtonText[1] = getMessage("no");

        if (iOptionType == JOptionPane.YES_NO_CANCEL_OPTION) {
          strDialogWarningButtonText[2] = getMessage("cancel");
        } 
        break;

      default:
        // nicht zu tun
    }   // switch

    int iresult = 
      JOptionPane.showOptionDialog(comp, strArrayMessages, getMessage((strTitleKey == null || strTitleKey.equals("")) 
            ? "attention" 
            : strTitleKey), iOptionType, iMessageType, null, strDialogWarningButtonText, strDialogWarningButtonText[0]);

    return iresult;
  }   // askQuestion
 
  /**
   * Ausgabe einer einfachen Meldung mit einem seperaten Dialog.
   * Der Benutzer kann nur die Meldung besttigen.
   *
   * @param comp Vaterkomponente
   * @param strMessage Schluessel fuer den Meldungtext
   * @param strTitle Schluessel fuer den Titeltext
   * @param iMessageType Welcher Messagetyp
   *
   * @see javax.swing.JOptionPane#showMessageDialog(Component, Object, String,int)
   */
  static public void showMessage(Component comp, String strMessage, 
                                 String strTitle, int iMessageType) {
    Object[]  strArrayMessages = getMessageObject(strMessage);

    JOptionPane.showMessageDialog(comp, strArrayMessages, strTitle, 
                                  iMessageType);
  } // showMessage

  
}

